from typing import Optional

import aiohttp

from hipcall_sdk.client.base import HipcallBaseClient
from hipcall_sdk.models import (
    CallDetailResponse,
    CallListResponse,
    TaskListResponse,
    TaskCreate,
    TaskResponse,
    TaskDetailResponse,
    CallAndBridgeResponse,
)


class AsyncClient(HipcallBaseClient):
    def __init__(self, api_key: str, base_url: Optional[str] = None):
        super().__init__(api_key, base_url)
        self.session: Optional[aiohttp.ClientSession] = None

    async def __aenter__(self):
        self.session = aiohttp.ClientSession(headers=self.get_default_headers())
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session:
            await self.session.close()

    async def get_call(self, call_id: str, date: str) -> CallDetailResponse:
        url = self.get_call_detail_url(call_id)
        params = {"date": date}
        async with self.session.get(url, params=params) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return CallDetailResponse(**data)

    async def get_calls(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "started_at.asc",
    ) -> CallListResponse:
        url = self.get_call_list_url()
        params = {"limit": limit, "offset": offset, "sort": sort}
        if q:
            params["q"] = q
        async with self.session.get(url, params=params) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return CallListResponse(**data)

    async def call_and_bridge(
        self,
        user_id: int,
        callee_number: str,
        ring_user_first: bool = True,
    ) -> CallAndBridgeResponse:
        url = self.get_call_and_bridge_url(user_id)
        data = {"callee_number": callee_number, "ring_user_first": ring_user_first}
        async with self.session.post(url, json=data) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return CallAndBridgeResponse(**data)

    async def get_tasks(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "id.asc",
    ) -> TaskListResponse:
        url = self.get_task_list_url()
        params = {"limit": limit, "offset": offset, "sort": sort}
        if q:
            params["q"] = q
        async with self.session.get(url, params=params) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return TaskListResponse(**data)

    async def create_task(self, task: TaskCreate) -> TaskResponse:
        url = self.get_task_list_url()
        data = task.model_dump(exclude_unset=True)
        async with self.session.post(url, json=data) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return TaskResponse(**data)

    async def get_task(self, task_id: int) -> TaskDetailResponse:
        url = self.get_task_detail_url(task_id)
        async with self.session.get(url) as response:
            content = await response.json()
            data = self.handle_response(response.status, content)
            return TaskDetailResponse(**data)



class HipcallAPIException(Exception):
    def __init__(self, status_code: int, content: dict):
        self.status_code = status_code
        self.content = content

    def __str__(self):
        return f"Unexpected error. Status: {self.status_code}, Content: #{self.content}"


class BadRequestException(HipcallAPIException):
    def __str__(self):
        return f"Bad request: {self.content["errors"]["detail"]}"


class UnauthorizedException(HipcallAPIException):
    def __str__(self):
        return "Invalid API key or unauthorized access"


class NotFoundException(HipcallAPIException):
    def __str__(self):
        return "The requested resource was not found"


class UnprocessableEntityException(HipcallAPIException):
    def __str__(self):
        errors = " | ".join(
            [f"{k}: {', '.join(v)} " for k, v in self.content["errors"]]
        )
        return f"The request was unprocessable: Errors: {errors}"



from typing import Optional

import requests

from hipcall_sdk.client.base import HipcallBaseClient
from hipcall_sdk.models import (
    CallDetailResponse,
    CallListResponse,
    TaskListResponse,
    TaskCreate,
    TaskResponse,
    TaskDetailResponse,
    CallAndBridgeResponse,
)


class Client(HipcallBaseClient):
    def __init__(self, api_key: str, base_url: Optional[str] = None):
        super().__init__(api_key, base_url)
        self.session = requests.Session()
        self.session.headers.update(self.get_default_headers())

    def get_call(self, call_id: str, date: str) -> CallDetailResponse:
        url = self.get_call_detail_url(call_id)
        params = {"date": date}
        response = self.session.get(url, params=params)
        data = self.handle_response(response.status_code, response.json())
        return CallDetailResponse(**data)

    def get_calls(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "started_at.desc",
    ) -> CallListResponse:
        url = self.get_call_list_url()
        params = {"limit": limit, "offset": offset, "sort": sort}
        if q:
            params["q"] = q
        response = self.session.get(url, params=params)
        data = self.handle_response(response.status_code, response.json())
        return CallListResponse(**data)

    def call_and_bridge(
        self,
        user_id: int,
        callee_number: str,
        ring_user_first: bool = True,
    ) -> CallAndBridgeResponse:
        url = self.get_call_and_bridge_url(user_id)
        data = {"callee_number": callee_number, "ring_user_first": ring_user_first}
        response = self.session.post(url, json=data)
        data = self.handle_response(response.status_code, response.json())
        return CallAndBridgeResponse(**data)

    def get_tasks(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "id.asc",
    ) -> TaskListResponse:
        url = self.get_task_list_url()
        params = {"limit": limit, "offset": offset, "sort": sort}
        if q:
            params["q"] = q
        response = self.session.get(url, params=params)
        data = self.handle_response(response.status_code, response.json())
        return TaskListResponse(**data)

    def create_task(self, task: TaskCreate) -> TaskResponse:
        url = self.get_task_list_url()
        response = self.session.post(url, json=task.model_dump(exclude_unset=True))
        data = self.handle_response(response.status_code, response.json())
        return TaskResponse(**data)

    def get_task(self, task_id: int) -> TaskDetailResponse:
        url = self.get_task_detail_url(task_id)
        response = self.session.get(url)
        data = self.handle_response(response.status_code, response.json())
        return TaskDetailResponse(**data)



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel


class Meta(BaseModel):
    count: int
    limit: int
    offset: int


class Call(BaseModel):
    uuid: str
    started_at: str
    ended_at: str
    direction: str
    call_duration: Optional[int] = None
    first_touch_duration: Optional[int] = None
    missing_call: Optional[bool] = None
    answered_at: Optional[str] = None
    bridged_at: Optional[str] = None


class CallDetail(BaseModel):
    uuid: str
    started_at: datetime
    ended_at: datetime
    direction: str
    call_duration: Optional[int] = None
    first_touch_duration: Optional[int] = None
    missing_call: Optional[bool] = None
    answered_at: Optional[datetime] = None
    bridged_at: Optional[datetime] = None
    caller_id: Optional[int] = None
    contact_id: Optional[int] = None
    credited: Optional[bool] = None
    related_id: Optional[int] = None
    related_type: Optional[str] = None
    channel_type: Optional[str] = None
    number_id: Optional[int] = None
    caller_number: Optional[str] = None
    voicemail_id: Optional[int] = None
    voicemail_type: Optional[str] = None
    hangup_by: Optional[str] = None
    missing_call_reason: Optional[str] = None
    caller_type: Optional[str] = None
    callee_id: Optional[int] = None
    voicemail_url: Optional[str] = None
    channel_id: Optional[int] = None
    user_id: Optional[int] = None
    callee_number: Optional[str] = None
    call_flow: Optional[str] = None
    record_url: Optional[str] = None
    callback_time: Optional[datetime] = None
    callback_cdr_uuid: Optional[str] = None
    callee_type: Optional[str] = None
    callback_user_id: Optional[int] = None


class CallListResponse(BaseModel):
    data: List[Call]
    meta: Meta


class CallDetailResponse(BaseModel):
    data: CallDetail


class CallAndBridge(BaseModel):
    id: str


class CallAndBridgeResponse(BaseModel):
    data: CallAndBridge


class Task(BaseModel):
    id: int
    name: str
    description: Optional[str] = None
    done: bool
    done_at: Optional[datetime] = None


class TaskListResponse(BaseModel):
    data: List[Task]
    meta: Meta


class TaskCreate(BaseModel):
    name: str
    description: Optional[str] = None
    assign_to_user_id: Optional[int] = None
    auto_assign_to_user: Optional[bool] = None
    company_ids: Optional[List[int]] = None
    contact_ids: Optional[List[int]] = None
    deal_ids: Optional[List[int]] = None
    due_date: Optional[datetime] = None
    priority: Optional[str] = None
    task_list_id: Optional[int] = None


class TaskResponse(BaseModel):
    data: Task


class TaskDetail(BaseModel):
    id: int
    name: str
    description: Optional[str] = None
    done: bool
    done_at: Optional[datetime] = None
    assign_to_user_id: Optional[int] = None
    auto_assign_to_user: Optional[bool] = None
    company_ids: Optional[List[int]] = None
    contact_ids: Optional[List[int]] = None
    deal_ids: Optional[List[int]] = None
    due_date: Optional[datetime] = None
    priority: Optional[str] = None
    task_list_id: Optional[int] = None


class TaskDetailResponse(BaseModel):
    data: TaskDetail



from abc import ABC, abstractmethod
from typing import Optional

from hipcall_sdk.constants import DEFAULT_HIPCALL_BASE_ADDRESS
from hipcall_sdk.exceptions import (
    UnauthorizedException,
    NotFoundException,
    UnprocessableEntityException,
    HipcallAPIException,
    BadRequestException,
)
from hipcall_sdk.models import (
    CallDetailResponse,
    CallListResponse,
    TaskListResponse,
    TaskResponse,
    TaskCreate,
    TaskDetailResponse,
    CallAndBridgeResponse,
)


EXCEPTION_MAP = {
    400: BadRequestException,
    401: UnauthorizedException,
    404: NotFoundException,
    422: UnprocessableEntityException,
}


class HipcallBaseClient(ABC):
    """
    Base class for Hipcall API clients.

    This class defines the interface for Hipcall API clients and provides
    some common functionality.

    Attributes:
        api_key (str): The API key used for authentication.
        base_url (str): The base URL for the Hipcall API, optional.
    """

    def __init__(self, api_key: str, base_url: Optional[str] = None):
        if not api_key:
            raise ValueError("API key must be provided.")

        self.api_key = api_key
        self.base_url = base_url or DEFAULT_HIPCALL_BASE_ADDRESS

    def get_default_headers(self) -> dict[str, str]:
        """Get the default headers for the Hipcall API client.

        :return: Dict of default headers
        """
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

    @staticmethod
    def handle_response(status_code: int, content: dict) -> dict:
        """Handle Hipcall API response status codes.

        :param status_code: HTTP response status code.
        :param content: HTTP response content.
        :return: Response dict if status is OK.
        :raises HipcallAPIException: If status code is not OK.
        """
        if status_code in [200, 201]:
            return content

        exc_class = EXCEPTION_MAP.get(status_code, HipcallAPIException)
        raise exc_class(status_code, content)

    def get_call_detail_url(self, call_id: str) -> str:
        """Get call detail URL for call ID.

        :param call_id: UUID of the call.
        :return: URL for fetching specific call detail.
        """
        return f"{self.base_url}/api/v3/calls/{call_id}"

    def get_call_list_url(self) -> str:
        """Get call list URL.

        :return: URL for fetching call list.
        """
        return f"{self.base_url}/api/v3/calls"

    def get_call_and_bridge_url(self, user_id) -> str:
        """Get call and bridge URL for given user ID.

        :param user_id: Database ID of the user.
        :return: URL for calling and bridging.
        """
        return f"{self.base_url}/api/v3/users/{user_id}/call"

    def get_task_list_url(self) -> str:
        """Get task list URL.

        :return: URL for fetching task list.
        """
        return f"{self.base_url}/api/v3/tasks"

    def get_task_detail_url(self, task_id) -> str:
        """Get task detail URL for given task ID.

        :param task_id: Database ID of the task.
        :return: URL for fetching task detail.
        """
        return f"{self.base_url}/api/v3/tasks/{task_id}"

    @abstractmethod
    def get_call(self, call_id: str, date: str) -> CallDetailResponse:
        """Get call details for given call ID and date.

        :param call_id: UUID of the call.
        :param date: Date of the call.
        :return: Pydantic model containing call details.
        """
        pass

    @abstractmethod
    def get_calls(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "started_at.asc",
    ) -> CallListResponse:
        """Get list of calls with query.

        :param limit: Limit of query. Min 1, Max 100. Defaults to 10.
        :param offset: Offset of the query. Defaults to 0.
        :param q: Search query which is searched at caller_number and callee_number.
            Don't add + or 00 as prefix just write number with country code.
        :param sort: Sort parameter. Defaults to "started_at.asc".
        :return: Pydantic model containing call list and metadata.
        """
        pass

    @abstractmethod
    def call_and_bridge(
        self,
        user_id: int,
        callee_number: str,
        ring_user_first: bool = True,
    ) -> CallAndBridgeResponse:
        """Call and bridge user and given number.

        :param user_id: Database ID of the caller user.
        :param callee_number: Callee number.
        :param ring_user_first: Rings user first if set to True. Defaults to True.
        :return: Pydantic model containing call & bridge response.
        """
        pass

    @abstractmethod
    def get_tasks(
        self,
        limit: int = 10,
        offset: int = 0,
        q: Optional[str] = None,
        sort: str = "id.asc",
    ) -> TaskListResponse:
        """Get list of tasks with query.

        :param limit: Limit of query. Min 1, Max 100. Defaults to 10.
        :param offset: Offset of the query. Defaults to 0.
        :param q: Search query which is searched name of the task.
        :param sort: Sort parameter. Defaults to "id.asc".
            Supported fields: id and name
            Ordering options: asc, asc_nulls_last, asc_nulls_first, desc,
            desc_nulls_last, desc_nulls_first
        :return: Pydantic model containing task list and metadata.
        """
        pass

    @abstractmethod
    def create_task(self, task: TaskCreate) -> TaskResponse:
        """Create new task on Hipcall.

        :param task: Pydantic model containing data for Task creation.
        :return: Pydantic model containing created task information.
        """
        pass

    @abstractmethod
    def get_task(self, task_id: int) -> TaskDetailResponse:
        """Get task information by ID.

        :param task_id: Database ID of the task.
        :return: Pydantic model containing task information.
        """
        pass
